(* Raw Disk Access, solyga@gmx.de, 2023-03-25 .. 2024-12-15 *)
MODULE RDA;

IMPORT IO;
IMPORT LC, Config, BDrvs;

(*# save, call( o_a_copy => off ) *)

VAR
  buf: ARRAY [0..BDrvs.lenSct-1] OF BYTE;
  drv: BDrvs.TId;


PROCEDURE Fill( VAR b: ARRAY OF BYTE; v: BYTE );
VAR i: CARDINAL;
BEGIN
  FOR i := 0 TO HIGH( b ) DO b[i] := v END;
END Fill;

PROCEDURE FillX( VAR b: ARRAY OF BYTE; x: ARRAY OF BYTE; w: CARDINAL );
VAR i: CARDINAL;
BEGIN
  FOR i := 0 TO HIGH( b ) DO b[i] := x[ i MOD w ] END;
END FillX;

PROCEDURE Copy( VAR b: ARRAY OF BYTE; x: ARRAY OF BYTE );
VAR n, i: CARDINAL;
BEGIN
  n := SIZE( x );
  IF n > SIZE( b ) THEN n := SIZE( b ) END;
  FOR i := 0 TO n - 1 DO b[i] := x[i] END;
END Copy;


PROCEDURE MkStr( VAR s: ARRAY OF CHAR; x: ARRAY OF BYTE );
BEGIN
  IF LC.MkStr( s, x ) THEN RETURN END;
  s[0] := '?';
  IF SIZE( s ) > 1 THEN s[1] := 0C END;
END MkStr;


PROCEDURE Dump( b: ARRAY OF BYTE; n: CARDINAL );
CONST
  max = 16;
VAR
  i: CARDINAL;
  c: CHAR;
BEGIN
  IF n > SIZE( b ) THEN n := SIZE( b ) END;
  IF n > max THEN n := max END;
  IF n <= 0 THEN RETURN END;
  c := IO.PrefixChar;
  IO.PrefixChar := '0';
  FOR i := 0 TO n - 1 DO
    IO.WrChar( ' ' );
    IO.WrShtHex( SHORTCARD(b[i]), 2 );
  END;
  IO.PrefixChar := c;
END Dump;


PROCEDURE Show();

  PROCEDURE ShowDrv( id: BDrvs.TVldId );
  BEGIN
    IO.WrStr( 'Drive ' );
    IO.WrCard( ORD( id ), 1 );
    IO.WrStr( ' len ' );
    IO.WrLngCard( BDrvs.Len( id ), 1 );
    IO.WrStr( ' = ' );
    IO.WrLngHex( BDrvs.Len( id ), 1 );
    IO.WrLn();
  END ShowDrv;

VAR
  id: BDrvs.TVldId;
BEGIN
  IF drv > BDrvs.idNull THEN
    ShowDrv( drv );
    RETURN;
  END;
  FOR id := MIN( BDrvs.TVldId ) TO MAX( BDrvs.TVldId ) DO
    ShowDrv( id );
  END;
END Show;

PROCEDURE Read();
VAR
  pos, cnt: LONGCARD;
  str: Config.TValStr;
BEGIN
  IF ( drv = BDrvs.idNull ) OR ( BDrvs.Len( drv ) <= 0 ) THEN
    IO.WrStr( 'Drive ' );
    IO.WrCard( ORD( drv ), 1 );
    IO.WrStr( ' does not exist' );
    IO.WrLn();
    RETURN;
  END;
  pos := Config.data.beg;
  cnt := Config.data.cnt;
  WHILE LC.Cmp( cnt, 0 ) > 0 DO
    IF ~ BDrvs.Read( drv, pos, buf, 0, 1 ) THEN
      MkStr( str, pos );
      IO.WrStr( 'Read error for drive ' );
      IO.WrCard( ORD( drv ), 1 );
      IO.WrStr( ' at sector ' );
      IO.WrStr( str );
      IO.WrLn();
      RETURN;
    END;
    IF Config.data.vrb > 0 THEN
      MkStr( str, pos );
      IO.WrStr( str );
      IO.WrStr( ':' );
      Dump( buf, 16 );
      IO.WrLn();
    END;
    LC.Inc( pos );
    LC.Dec( cnt );
  END;
END Read;

PROCEDURE Write();
VAR
  pos, cnt: LONGCARD;
  str: Config.TValStr;
BEGIN
  IF ( drv = BDrvs.idNull ) OR ( BDrvs.Len( drv ) <= 0 ) THEN
    IO.WrStr( 'Drive ' );
    IO.WrCard( ORD( drv ), 1 );
    IO.WrStr( ' does not exist' );
    IO.WrLn();
    RETURN;
  END;
  FillX( buf, Config.data.val, Config.data.wid );
  pos := Config.data.beg;
  cnt := Config.data.cnt;
  WHILE LC.Cmp( cnt, 0 ) > 0 DO
    IF ~ BDrvs.Write( drv, pos, buf, 0, 1 ) THEN
      MkStr( str, pos );
      IO.WrStr( 'Write error for drive ' );
      IO.WrCard( ORD( drv ), 1 );
      IO.WrStr( ' at sector ' );
      IO.WrStr( str );
      IO.WrLn();
      RETURN;
    END;
    IF Config.data.vrb > 0 THEN IO.WrChar( '.' ) END;
    LC.Inc( pos );
    LC.Dec( cnt );
  END;
  IF Config.data.vrb > 0 THEN IO.WrLn() END;
END Write;

PROCEDURE Sign();
VAR
  pos, cnt: LONGCARD;
  str: Config.TValStr;
BEGIN
  IF ( drv = BDrvs.idNull ) OR ( BDrvs.Len( drv ) <= 0 ) THEN
    IO.WrStr( 'Drive ' );
    IO.WrCard( ORD( drv ), 1 );
    IO.WrStr( ' does not exist' );
    IO.WrLn();
    RETURN;
  END;
  Fill( buf, 0 );
  pos := Config.data.beg;
  cnt := Config.data.cnt;
  WHILE LC.Cmp( cnt, 0 ) > 0 DO
    Copy( buf, pos );
    IF ~ BDrvs.Write( drv, pos, buf, 0, 1 ) THEN
      MkStr( str, pos );
      IO.WrStr( 'Write error for drive ' );
      IO.WrCard( ORD( drv ), 1 );
      IO.WrStr( ' at sector ' );
      IO.WrStr( str );
      IO.WrLn();
      RETURN;
    END;
    IF Config.data.vrb > 0 THEN IO.WrChar( '.' ) END;
    LC.Inc( pos );
    LC.Dec( cnt );
  END;
  IF Config.data.vrb > 0 THEN IO.WrLn() END;
END Sign;


BEGIN
  Fill( buf, 0 );
  IF Config.data.drv > ORD( MAX(BDrvs.TId) ) THEN
    IO.WrStr( 'Invalid drive number ' );
    IO.WrShtCard( Config.data.drv, 1 );
    IO.WrLn();
    RETURN;
  END;
  drv := BDrvs.TId( Config.data.drv );

  CASE Config.data.act OF
  | Config.actShow : Show();
  | Config.actRead : Read();
  | Config.actWrite: Write();
  | Config.actSign : Sign();
  ELSE
    IO.WrStr( 'Action not implemented' );
    IO.WrLn();
  END;
END RDA.
